//--------------------------------------------------------
// StormExTractor by unk1024
//

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <direct.h>
#include <string>

#include "zconf.h"
#include "zlib.h"

using namespace std;

#pragma comment (lib, "zlib.lib")

#define MAX_FILES						2048

#define USAGE \
	"StormExTractor by unk1024\n" \
	"\n" \
	"Usage: %s FILE_NAME\n" \
	"\n"

#define ZLIB_CHECK_ERR(err, msg) \
	{ \
		if (err != Z_OK) \
		{ \
			printf ("%s error: %d\n", msg, err); \
			exit (1); \
		} \
	}

std::string extract_filename (std::string filename)
{
	size_t i = filename.find_last_of ('\\') + 1;
	return filename.substr (i, filename.length ());
}

std::string extract_dir (std::string filename)
{
	size_t i = filename.find_last_of ('\\');
	return filename.substr (0, i);
}

void mkdir_recursive (std::string dir, std::string root)
{
	if (dir.find_first_of ('.') != -1)
		return;

	std::string full;
	size_t i = dir.find_first_of ('\\');
	full = root + "\\" + dir.substr (0, i);
	_mkdir (full.c_str ());

	if (i < dir.length ())
		mkdir_recursive (dir.substr (i + 1, dir.length ()), full);
}

int main (int argc, char * args [])
{
	if (argc < 2)
	{
		std::string filename = args [0];
		filename = extract_filename (filename);
		
		printf (USAGE, filename.c_str ());
		return 0;
	}

	std::string filename = (args [1]) ? args [1] : "";

	if (filename.empty ())
		filename = "stormex2.dll";

	struct fat_header_t
	{
		int res_offset;
		int zero1;
		int ft_offset;
		int zero2;
		int ft_size;
		int zero3;
	};

	struct fat_table_fileinfo_t
	{
		int filename_len;
		char filename [MAX_PATH];
		int size;
		int zero1;
		int offset;
		int zero2;
	};
	
	fat_table_fileinfo_t * fileinfos = (fat_table_fileinfo_t *) new fat_table_fileinfo_t [MAX_FILES];

	unsigned int cur_offset, len, i = 0, num_files;
	int err;

	FILE * fp = fopen (filename.c_str (), "rb");

	if (!fp)
	{
		printf ("Could'nt open file \"%s\".", filename.c_str ());
		return 0;
	}

	cur_offset = ftell (fp);
	fseek (fp, 0, SEEK_END);
	
	len = ftell (fp);

	fseek (fp, cur_offset, SEEK_SET);
	fseek (fp, len - sizeof (fat_header_t), SEEK_SET);

    fat_header_t fat_header;
	fread (&fat_header, 1, sizeof (fat_header_t), fp);

	fseek (fp, fat_header.ft_offset, SEEK_SET);

	while (ftell (fp) < (len - sizeof (fat_header_t)))
	{
		if (i > MAX_FILES)
			break;

		memset (&fileinfos [i], 0, sizeof (fat_table_fileinfo_t));

		fread (&fileinfos [i].filename_len, 1, sizeof (fileinfos [i].filename_len), fp);
		fread (fileinfos [i].filename, 1, fileinfos [i].filename_len, fp);
		fileinfos [i].filename [fileinfos [i].filename_len] = '\0';

		fread (&fileinfos [i].size, 1, sizeof (fileinfos [i].size), fp);
		fread (&fileinfos [i].zero1, 1, sizeof (fileinfos [i].zero1), fp);
		fread (&fileinfos [i].offset, 1, sizeof (fileinfos [i].offset), fp);
		fread (&fileinfos [i].zero2, 1, sizeof (fileinfos [i].zero2), fp);

		i++;
	}
	
	num_files = i;

	zlib_version;

	_mkdir ("PROGRAM\\");

	byte * compr, * uncompr;

	for (int i = 0; i < num_files; i++)
	{
		uLong comprLen = (uLong) fileinfos [i].size;
		uLong uncomprLen = comprLen;

		std::string outfile = "PROGRAM\\";
		outfile += fileinfos [i].filename;

		compr = (byte *) calloc ((uInt) comprLen, 1);
		uncompr = (byte *) calloc ((uInt) uncomprLen, 1);

		fseek (fp, fileinfos [i].offset, SEEK_SET);
		fread (compr, 1, fileinfos [i].size, fp);

		err = uncompress (uncompr, &uncomprLen, compr, comprLen);

		mkdir_recursive (extract_dir (fileinfos [i].filename), ".\\PROGRAM");

		FILE * fp2 = fopen (outfile.c_str (), "wb");
		fwrite (uncompr, 1, uncomprLen, fp2);
		fclose (fp2);

		free (uncompr);
		free (compr);
	}

	fclose (fp);

	free (fileinfos);

	return 0;
}
